// Fill out your copyright notice in the Description page of Project Settings.

#include "PilotCamera.h"
#include "MonolithPlayerController.h"
#include "MonolithCharacter.h"
#include "MoveState.h"

void UPilotCamera::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	if (TickType == LEVELTICK_ViewportsOnly
		|| (!isMoveStateSet() && !TrySetMoveState())) return;

	// TODO - Set FOV based on Velocity mag
	UpdateFOV(DeltaTime);

	// TODO - Add Camera Roll if Strafing
	UpdateRoll(DeltaTime);
	// if (MoveState->Is(STRAFE_L)) UE_LOG(LogTemp, Warning, TEXT("Strafing Left"));
	// if (MoveState->Is(STRAFE_R)) UE_LOG(LogTemp, Warning, TEXT("Strafing Right"));
	// if (MoveState->Is(SPRINT))   UE_LOG(LogTemp, Warning, TEXT("Sprinting"));
	// if (MoveState->Is(GROUNDED)) UE_LOG(LogTemp, Warning, TEXT("On the floor"));
	// if (MoveState->Is(FORWARD))  UE_LOG(LogTemp, Warning, TEXT("Going FORWARD"));
}

bool UPilotCamera::TrySetMoveState()
{
	AMonolithCharacter* PlayerChar = Cast<AMonolithCharacter>
		(
			Cast<AMonolithPlayerController>(GetWorld()->GetFirstPlayerController())->GetPawn()
		);
	if ( PlayerChar == nullptr) return false;
	MoveState  = &PlayerChar->MoveState;
	return MoveState != nullptr;
}

void UPilotCamera::UpdateFOV(float DeltaTime)
{
	const float targetFOV = GetTargetFOV();
	if (FieldOfView == targetFOV) return;
	
	float linearModifier = 1 - (FMath::Abs(targetFOV - FieldOfView)) / targetFOV;
	float rateOfChange = RateOfChange * DeltaTime * linearModifier;
	bool increasing = targetFOV > FieldOfView;
	rateOfChange *= increasing ? 1 : -1;
	
	float newFOV = FieldOfView + rateOfChange;
	if ((increasing && newFOV > targetFOV) || (!increasing && newFOV < targetFOV))
	{
		newFOV = targetFOV;
	}
	SetFieldOfView(newFOV);
}

void UPilotCamera::UpdateRoll(float DeltaTime)
{
	FRotator rot = GetRelativeRotation();
	if (MoveState->Is(STRAFE_R))
	{
		TargetRoll = MaxStrafeRoll;
	}
	else if (MoveState->Is(STRAFE_L))
	{
		TargetRoll = -MaxStrafeRoll;
	}
	else
	{
		TargetRoll = 0;
	}
	float rateOfChange = RollRateOfChange * DeltaTime;
	bool increasing = TargetRoll > rot.Roll;
	rateOfChange *= increasing ? 1 : -1;
	rot.Roll += rateOfChange;
	if ((increasing && rot.Roll > TargetRoll) || (!increasing && rot.Roll < TargetRoll))
	{
		rot.Roll = TargetRoll;
	}
	
	SetRelativeRotation(rot);
}

